import React, { LegacyRef, useEffect, useRef, useState } from 'react';
import * as d3 from 'd3';

type BarPlotProps = {
  data: { category: string; count: number }[];
  numBins: number;
  typeBarPlot: 'numerical' | 'categorical';
};

export const BarPlot = ({ typeBarPlot: typeBarplot, numBins, data }: BarPlotProps) => {
  const svgRef = useRef<SVGSVGElement | null>(null);

  useEffect(() => {
    if (!svgRef.current) return;

    const widthSVG: number = +svgRef.current.clientWidth;
    const heightSVG: number = +svgRef.current.clientHeight;

    const marginPercentage = { top: 0.15, right: 0.15, bottom: 0.15, left: 0.15 };
    const margin = {
      top: marginPercentage.top * heightSVG,
      right: marginPercentage.right * heightSVG,
      bottom: marginPercentage.bottom * heightSVG,
      left: marginPercentage.left * heightSVG,
    };

    const svgPlot = d3.select(svgRef.current);

    svgPlot.selectAll('*').remove();
    const groupMargin = svgPlot.append('g').attr('transform', `translate(${margin.left},${margin.top})`);
    const widthSVGwithinMargin: number = widthSVG - margin.left - margin.right;
    const heightSVGwithinMargin: number = heightSVG - margin.top - margin.bottom;

    // Barplot for categorical data
    if (typeBarplot == 'categorical') {
      // Data processing
      const dataFiltered = data.filter((item) => item.category !== undefined);
      const dataSorted = dataFiltered.sort((a, b) => b.count - a.count);
      const maxCount = d3.max(dataSorted, (d) => d.count) ?? 1;

      const xScale = d3
        .scaleBand()
        .range([0, widthSVGwithinMargin])
        .domain(
          dataSorted.map(function (d) {
            return d.category;
          })
        )
        .padding(0.2);

      const yScale = d3.scaleLinear().domain([0, maxCount]).range([heightSVGwithinMargin, 0]).nice();

      groupMargin
        .selectAll<SVGRectElement, d3.Bin<string, number>>('barplotCats')
        .data(dataSorted)
        .enter()
        .append('rect')
        .attr('x', (d) => xScale(d.category) || 0)
        .attr('y', (d) => yScale(d.count))
        .attr('width', xScale.bandwidth())
        .attr('height', (d) => heightSVGwithinMargin - yScale(d.count))
        .attr('class', 'fill-primary stroke-white');

      const yAxis = d3.axisLeft(yScale).ticks(5);
      groupMargin.append('g').call(yAxis);

      // Barplot for numerical data
    } else {
      const dataCount = data.map((obj) => obj.count);

      const extentData = d3.extent(dataCount);
      const [min, max] = extentData as [number, number];

      const xScale = d3.scaleLinear().range([0, widthSVGwithinMargin]).domain([min, max]).nice();

      // d3.histogram -> deprecated
      // On d3.bin(), .thresholds(x.ticks(numBins)). Creates artifacts: not full rects on the plot
      // CHECK THIS: https://dev.to/kevinlien/d3-histograms-and-fixing-the-bin-problem-4ac5

      const rangeTH: number[] = d3.range(min, max, (max - min) / numBins);

      if (rangeTH.length != numBins) {
        rangeTH.pop();
      }

      const histogram = d3
        .bin()
        .value(function (d) {
          return d;
        })
        .domain([min, max])
        .thresholds(rangeTH);

      const bins = histogram(dataCount);

      const yScale = d3
        .scaleLinear()
        .range([heightSVGwithinMargin, 0])
        .domain(d3.extent(bins, (d) => d.length) as [number, number]);

      groupMargin
        .selectAll<SVGRectElement, d3.Bin<number, number>>('barplotbins')
        .data(bins)
        .enter()
        .append('rect')
        .attr('x', 1)
        .attr('class', 'fill-primary stroke-white')
        .attr('transform', (d) => 'translate(' + xScale(d.x0 || 0) + ',' + yScale(d.length) + ')')
        .attr('width', (d) => xScale(d.x1 || 0) - xScale(d.x0 || 0) - 1)
        .attr('height', (d) => heightSVGwithinMargin - yScale(d.length));

      const xAxis = d3.axisBottom(xScale).ticks(5);

      groupMargin
        .append('g')
        .attr('transform', 'translate(0,' + heightSVGwithinMargin + ')')
        .call(xAxis);
    }

    svgPlot.selectAll('.tick text').attr('class', 'font-inter text-primary font-semibold').style('stroke', 'none');

    groupMargin
      .append('rect')
      .attr('x', 0.0)
      .attr('y', 0.0)
      .attr('width', widthSVGwithinMargin)
      .attr('height', heightSVGwithinMargin)
      .attr('rx', 0)
      .attr('ry', 0)
      .attr('class', 'fill-none stroke-secondary');

    svgPlot
      .append('rect')
      .attr('x', 0.0)
      .attr('y', 0.0)
      .attr('width', widthSVG)
      .attr('height', heightSVG)
      .attr('rx', 0)
      .attr('ry', 0)
      .attr('class', 'fill-none stroke-secondary');
  }, [data]);

  return (
    <div className="svg">
      <svg ref={svgRef} className="container" width="100%" height="100%"></svg>
    </div>
  );
};

export default BarPlot;
